import {parse_spec, type Variant} from "gfonts-js-library/src/css2";
import {retrieve_font_metadata} from "gfonts-js-library/src/metadata"

interface MirrorVariant {
    path: string,
    name: string,
    weight: number,
    style: string,
}

type MirrorFamily = MirrorVariant[]
type MirrorMetadata = MirrorFamily[]

interface MirrorVariantUnifiedWeights {
    path: string,
    name: string,
    weights: number[],
    style: string
}

type MirrorFamilyUnifiedWeights = MirrorVariantUnifiedWeights[]

function to_array(v) {
    if (Array.isArray(v)) {
        return v
    } else {
        return [v]
    }
}
export async function css2_to_cssfile(css2_url, mirror): Promise<string> {
    let url = css2_url
        .replace("https://fonts.googleapis.com/css2", "")
        .replaceAll("&display=swap", "")
        .replaceAll("&", "?")

    let rfamilies = url.split("?family=")
    rfamilies.shift()
    let mirror_meta = await Promise.all(rfamilies
        .map(v => v.replace("?family=", ""))
        .map(parse_spec)
        .map(async (v) => {
            let {url, metadata} = await retrieve_font_metadata(v.font, mirror)
            let fonts = []
            for (let variant of v.variants) {
                let { weight, style }: Variant = variant
                let rfonts = to_array(metadata.fonts)
                let matched = false
                for (let font of rfonts) {
                    if (font.weight === weight && matched === false) {
                        if (style === "italic") {
                            fonts.push({
                                name: metadata.name,
                                path: url + "/" + font.filename,
                                weight,
                                style,
                            })
                            matched = true
                        }
                    }
                }

                if (matched === false) {
                    fonts.push({
                        name: metadata.name,
                        path: url + "/" + rfonts[0].filename,
                        weight,
                        style,
                    })
                }
            }

            return fonts
        }))

    return mirror_metadata_to_css(mirror_meta)
}

function mirror_metadata_to_css(mirror_meta: MirrorMetadata): string {
    let css = ""
    for (let family of mirror_meta) {
        for (let variant of unify_weights(family)) {
            css +=
`@font-face {
    font-family: "${variant.name}";
    src: url("${variant.path}");
    font-style: ${variant.style};
    font-weight: ${variant.weights.join(" ")};
}

`
        }
    }

    return css.trim()
}

function unify_weights(family: MirrorFamily): MirrorFamilyUnifiedWeights {
    let unified: MirrorFamilyUnifiedWeights = []
    for (let variant of family) {
        let added = false
        for (let uni_val of unified) {
            if (uni_val.name === variant.name && uni_val.style === variant.style && uni_val.path === variant.path) {
                uni_val.weights.push(variant.weight)
                added = true
            }
        }

        if (added === false) {
            unified.push({
                name: variant.name,
                style: variant.style,
                path: variant.path,
                weights: [variant.weight],
            })
        }
    }

    return unified
}
